@@ -46,6 +46,7 @@ Plist parsing example:
 
 from collections import namedtuple
 from cStringIO import StringIO
+import calendar
 import datetime
 import math
 import plistlib
@@ -61,8 +62,10 @@ __all__ = [
 apple_reference_date_offset = 978307200
 
 class Uid(int):
-    """Wrapper around integers for representing UID values."""
-    pass
+    """Wrapper around integers for representing UID values. This
+       is used in keyed archiving."""
+    def __repr__(self):
+        return "Uid(%d)" % self
 
 class Data(str):
     """Wrapper around str types for representing Data values."""
@@ -88,6 +91,7 @@ def readPlist(pathOrFile):
         result = reader.parse()
     except NotBinaryPlistException, e:
         try:
+            pathOrFile.seek(0)
             result = plistlib.readPlist(pathOrFile)
         except Exception, e:
             raise InvalidPlistException(e)
@@ -257,17 +261,7 @@ class PlistReader(object):
         result = 0
         original_offset = self.currentOffset
         data = self.contents[self.currentOffset:self.currentOffset+bytes]
-        # 1, 2, and 4 byte integers are unsigned
-        if bytes == 1:
-            result = unpack('>B', data)[0]
-        elif bytes == 2:
-            result = unpack('>H', data)[0]
-        elif bytes == 4:
-            result = unpack('>L', data)[0]
-        elif bytes == 8:
-            result = unpack('>q', data)[0]
-        else:
-            raise InvalidPlistException("Encountered integer longer than 8 bytes.")
+        result = self.getSizedInteger(data, bytes)
         self.currentOffset = original_offset + bytes
         return result
     
@@ -387,16 +381,14 @@ class PlistWriter(object):
         # A set of all the uniques which have been computed.
         self.computedUniques = set()
         # A list of all the uniques which have been written.
-        self.writtenReferences = []
+        self.writtenReferences = {}
         # A dict of the positions of the written uniques.
         self.referencePositions = {}
         
     def positionOfObjectReference(self, obj):
         """If the given object has been written already, return its
            position in the offset table. Otherwise, return None."""
-        if obj in self.writtenReferences:
-            return self.writtenReferences.index(obj)
-        return None
+        return self.writtenReferences.get(obj)
         
     def writeRoot(self, root):
         """
@@ -534,7 +526,7 @@ class PlistWriter(object):
         """
         position = self.positionOfObjectReference(obj)
         if position is None:
-            self.writtenReferences.append(obj)
+            self.writtenReferences[obj] = len(self.writtenReferences)
             output += self.binaryInt(len(self.writtenReferences) - 1, bytes=self.trailer.objectRefSize)
             return (True, output)
         else:
@@ -572,7 +564,7 @@ class PlistWriter(object):
         elif isinstance(obj, Uid):
             size = self.intSize(obj)
             output += pack('!B', (0b1000 << 4) | size - 1)
-            output += self.binaryInt(Uid)
+            output += self.binaryInt(obj)
         elif isinstance(obj, (int, long)):
             bytes = self.intSize(obj)
             root = math.log(bytes, 2)
@@ -583,7 +575,7 @@ class PlistWriter(object):
             output += pack('!B', (0b0010 << 4) | 3)
             output += self.binaryReal(obj)
         elif isinstance(obj, datetime.datetime):
-            timestamp = time.mktime(obj.timetuple())
+            timestamp = calendar.timegm(obj.utctimetuple())
             timestamp -= apple_reference_date_offset
             output += pack('!B', 0b00110011)
             output += pack('!d', float(timestamp))
@@ -612,7 +604,7 @@ class PlistWriter(object):
                     (isNew, output) = self.writeObjectReference(objRef, output)
                     if isNew:
                         objectsToWrite.append(objRef)
-                for objRef in obj:
+                for objRef in objectsToWrite:
                     output = self.writeObject(objRef, output, setReferencePosition=True)
             elif isinstance(obj, dict):
                 output += proc_variable_length(0b1101, len(obj))
@@ -637,9 +629,11 @@ class PlistWriter(object):
     def writeOffsetTable(self, output):
         """Writes all of the object reference offsets."""
         all_positions = []
-        for obj in self.writtenReferences:
-            position = self.referencePositions.get(obj, None)
-            if position == None:
+        writtenReferences = self.writtenReferences.items()
+        writtenReferences.sort(lambda x,y: cmp(x[1], y[1]))
+        for obj,order in writtenReferences:
+            position = self.referencePositions.get(obj)
+            if position is None:
                 raise InvalidPlistException("Error while writing offsets table. Object not found. %s" % obj)
             output += self.binaryInt(position, self.trailer.offsetSize)
             all_positions.append(position)
